/*
 * arealight.h
 * Definitions for area lights that use path tracing
 */

#ifndef __AREALIGHT_H
#define __AREALIGHT_H

// Possible orientations of an area light
#define POSITIVE_X 0
#define NEGATIVE_X 1
#define POSITIVE_Y 2
#define NEGATIVE_Y 3
#define POSITIVE_Z 4
#define NEGATIVE_Z 5

#include "object.h"
#include "primitive.h"
#include "surface.h"
#include "ray.h"
#include "csg.h"

/*
 * An area light source is NOT a light source but is an object ! Revelation !
 */
class AreaLight: public Primitive
{
	public:
	
	int orientation ; // As specified in the definitions above
	double min1, min2, max1, max2 ; // Cyclic endpt coords
	double main ; // Main coordinate - x, y or z depending or orientation

	AreaLight( Surface*, int, double, double, double, double, double ) ;
	AreaLight( const AreaLight& ) ; // Copy constructor
	virtual bool intersect( Ray& ) ;
	virtual bool shadow_intersect( Ray& ) ;
	virtual void computeBounds( bVolume& b ) ;
	virtual Color shade( const Ray&, int depth ) ;
	virtual ~AreaLight() {}

	virtual vector <CSG_Interval*> compute_intervals( const Ray& ) ; // Will be dummy, at least for now
};

#endif
