// inverse.cpp - Code to compute matrix inverse using LUP Decomposition
// Algorithm as detailed in CLR Chapter 29 on Matrix algorithms

#include <math.h>
#include <iostream>
#include <assert.h>

using namespace std;

/*
void print_matrix( double A[4][4] ) 
{	
	int i, j ;	
	for( i=0; i<4; i++ )
	{
		for( j=0; j<4; j++ )
			cout<<A[i][j]<<" " ;
		cout<<endl ;
	}
	cout<<endl ;
}

*/

void LUP_Solve( double** L, double** U, int* pi, double* b, double *x )
{
	double y[4], sum_y, sum_x ;
	int n = 4 ;
	int i, j ;

	for( i=0; i<n; i++ )
	{
		sum_y  = 0 ;
		for( j=0; j<=i-1; j++ )
		{
			sum_y += L[i][j]*y[j] ;
		}
		y[i] = b[ pi[i] ] - sum_y ;
	}

	for( i=n-1; i>=0; i-- )
	{
		sum_x = 0 ;
		for( j=i+1; j<n; j++ )
		{
			sum_x += U[i][j]*x[j] ;
		}
		
		x[i] = ( y[i] - sum_x ) / U[i][i] ;
	}
}


void LUP_Decomposition( double** A, double** L, double** U, int* pi )
{
	int n = 4 ;
	int i, j, k, index, swap_int ;
	double p, swap ;

	for( i=0; i<n; i++ )
	{
		pi[i] = i ;
		for( j=0; j<n; j++ )
		{
			L[i][j] = U[i][j] = 0 ;
		}
		L[i][i] = 1 ;
	}
	
	for( k=0; k<n-1; k++ )
	{
		p = 0 ;
		for( i = k; i<n; i++ )
		{
			if( fabs( A[i][k] ) > p )
			{
				p = fabs( A[i][k] ) ;
				index = i ;
			}
		}

		assert( p!= 0 ) ;
		
		// swap pi[k] and pi[index] 
		swap_int = pi[k] ;
		pi[k] = pi[index] ;
		pi[index] = swap_int ;

		for( i=0; i<n; i++ )
		{
			swap = A[k][i] ;
			A[k][i] = A[index][i] ;
			A[index][i] = swap ;
		}

		for( i=k+1; i<n; i++ )
		{
			A[i][k] /= A[k][k] ;
			for( j=k+1; j<n; j++ )
				A[i][j] -= A[i][k]*A[k][j] ;
			
		}
	}

	for( i=0; i<n; i++ )
	{
		for( j=0; j<n; j++ )
		{
			if( i>j )	
				L[i][j] = A[i][j] ;
			else    U[i][j]	= A[i][j] ;
		}

	}
}
		
// The actual routine to invert a 4x4 matrix that will be invoked from the ray tracer (primitive.cpp)
void matrix_invert( double **M, double inverse[4][4] )
{
	double b[4], x[4] ;
	double** L, **U; 
	int *pi ;
	
	int i, j ;
	L = new double*[4] ; U = new double*[4] ; 
	pi = new int[4] ;
	
	for( i = 0 ; i < 4 ; i++ ) {
		L[i] = new double[4] ;
		U[i] = new double[4] ;
	}

	for( i=0; i<4; i++ )
		b[i] = 0 ;
/*
	cout<<"A:\n";
	print_matrix( M ) ;
*/
	LUP_Decomposition( M, L, U, pi ) ;
/*	cout<<"P:\n";
	print_matrix( P ) ;

	cout<<"L:\n";
	print_matrix( L ) ;

	cout<<"U:\n";
	print_matrix( U ) ;
*/
	for( i=0; i<4; i++ )
	{
		b[i] = 1 ;
		LUP_Solve( L, U, pi, b, x ) ;
		for( j=0; j<4; j++ )
			inverse[j][i] = x[j] ;
		b[i] = 0 ;
	}
	
	for( i = 0 ; i < 4 ; i++ ) {
		delete [] L[i] ;
		delete [] U[i] ;
	}

	delete [] L ; delete [] U ; delete [] pi ;
}

/*
int main()
{
	double A[4][4], inverse[4][4] ;
	int i, j ;

	for( i=0; i<4; i++ )
		for( j=0; j<4; j++ )
			cin>>A[i][j] ;
	
	matrix_invert( A, inverse ) ;
	print_matrix( inverse ) ;
}

*/
