// class declarations for Light class - light.h representing a source of light

#ifndef __LIGHT_H
#define __LIGHT_H

#include "vector3D.h"

// types of light sources
#define AMBIENT 	0
#define DIRECTIONAL     1
#define POINT		2
#define AREA		3

// Note - area and more complex lights are always modelled as objects with emissive surfaces

/*
 * Generic light class
 */
class Light
{
	public:
	
	double red, green, blue ; // intensity of the source in rgb 
	virtual int getLightType() = 0 ; // Gets the light type for the light
	virtual Vector3D get_light_vector() = 0 ; // Gets vector for illumination calculations
};

/*
 * Point light source
 */
class PointLight : public Light
{
	public:	
	Vector3D light_vector ; // coordinates of a point source
	
	// Constructor
	PointLight( Vector3D, double, double, double ) ; // Constructor for a point light source
	
	virtual int getLightType() { return POINT ; }
	virtual Vector3D get_light_vector() { return light_vector ; }
};

/*
 * Ambient light
 */
class AmbientLight: public Light
{
	public:
	// Constructor
	AmbientLight( double, double, double ) ; 
	
	virtual int getLightType() { return AMBIENT ; }
	virtual Vector3D get_light_vector() { return Vector3D(0,0,0) ; } // Not important 
};

/*
 * Directional light source
 */
class DirectionalLight: public Light
{
	public:
	Vector3D light_vector ; // direction of this light

	// Constructor
	DirectionalLight( Vector3D, double, double, double ) ;

	virtual int getLightType() { return DIRECTIONAL ; }
	virtual Vector3D get_light_vector() { return light_vector ; }
};

#endif
