// Abstract graphics object class - object.h

#ifndef __OBJECT_H
#define __OBJECT_H

#include "vector3D.h"
#include "light.h"
#include "color.h"
#include "ray.h"

#include <vector>

using std::vector ;

class Ray ;	// Declared before definition - opaque reference
class Surface ; // Declared before definition - opaque reference
class bVolume ; // Declared before definition - opaque reference

// This is an abstract class that can serve as an interface for any object, be it a sphere, teapot or evil algebraic surface
// It applies also to non primitives like CSG objects
class Object
{
	public:
	// Some virtual functions that will be late bound as usual :)
	virtual bool intersect( Ray& r ) = 0 ; // Intersects this object with a ray
	virtual bool shadow_intersect( Ray& r ) = 0 ; // Intersects with a shadow ray
	virtual void computeBounds( bVolume& b ) = 0 ; // Compute bounding volume for the object
	virtual Color shade( const Ray& r, int depth ) = 0 ; // Shades this object by passing parameters to the surface shader
	virtual void translate( double, double, double ) = 0 ; // Translates this object
	virtual void scale( double, double, double ) = 0 ; // Scales this object
	virtual void rotate( char, double ) = 0 ; // Rotates this object
	virtual void compute_inverse( void ) = 0 ; // Freezes this object by computing inverse transform
	virtual void setSurface( Surface* ) = 0 ; // Sets new surface properties for this object
	virtual Surface* getSurface() = 0 ; // Gets the surface associated with this object
};

#endif
