// Class declaration for the ray class - represents any ray traced in the ray tracer along with its primary object

#ifndef __RAY_H
#define __RAY_H

#define MAX_LENGTH 10e8

#include <vector>	// For STL vector

#include "vector3D.h"
#include "light.h"


using std::vector ;

class Object ; // Forward declaration
class Primitive ; // Forward declaration and opaque pointer

class Ray 
{

	public:
	Vector3D origin, direction; // The position vector of the origin and the direction vector of the ray
	double length;	// The distance of the closest object intersected so far
	Object* object;	// The closest object intersected so far
	
	// New parameter required for refraction
	double n ; // Refractive index of the medium in which this ray travels
	bool inside ; // Ray originates inside an object - to avoid generating reflected rays
	
	// New parameter added for CSG support
	Primitive* intersector ; // The primtive of intersection of the ray with the object
	
	// Parametric coordinates in object coordinate system for triangle interpolation / textures / both
	double u, v ; 
	
	Ray(Vector3D eye, Vector3D dir) ; // Creates a new ray from the eye along direction dir
	Ray() { } // Default constructor
	bool trace() ; // Traces this ray
	bool shadow_trace() ; // Traces this shadow ray (a little faster than trace())
	Color shade( int depth ) ; // A wrapper for the shade() method of the object
};

#endif
