/*
 * supersample.h
 * Definitions for supersample.cpp
 */

#include "color.h"
#include "ppm.h"
#include "surface.h"

#include <math.h>

#include <map>
using std::map ;

/* Defines resolution of a pixel */
#define RESOLUTION 0.60

/* Defines the quality of antialiasing - The RGB color difference required to trigger a subdivision
 * The lower the value of AA_QUALITY, the higher the quality of antialiasing ( and the slower the rendering :( )
 */
#define AA_QUALITY 0.1

/* Defines the maximum depth upto which to subdivide
 * Helps to avoid exponential explosion of recursion and resultant blowup of stack space usage
 * This parameter can (possibly) negate the effect of the previous parameter
 * So set it very carefully
 */
#define MAX_AA_DEPTH 2

struct Pixel {
		double i, j, size ;
};

/* Comparsion class for pixels
 * Defines the equality of two pixels for the purpose 
 * of lookup of memoized values
 * Need to check if use of pixel size is appropriate TODO
 */
class compare_pixel {
	public:
	bool operator () ( const Pixel& p1, const Pixel& p2 )
	{
		return ( fabs( p1.i - p2.i ) < DELTA && fabs( p1.j - p2.j ) < DELTA ) ;
	}
};


/*
 * Recursive adaptive subdivision routine declaration
 * Defined in supersample.cpp
 */
Color adaptive_subdivide( Pixel p, Color corner_colors[4], int depth = 0 ) ;
