/*
 * texture.cpp - Texture mapping implementation
 */

#include "ppm.h"
#include "texture.h"
#include "color.h"

#include <stdio.h>
#include <math.h> // For floor(), ceil()

// Loads the texture from a ppm file
void Texture::loadFromPPM( char* TextureFileName )
{
		textureMap = ImageRead( TextureFileName ) ;
		printf("\nLoaded texture map from %s", TextureFileName ) ;
}

// Auxiliary method for convenience
Color getColorAtPixel( Image* image, int x, int y )
{
		Color result( 0, 0, 0 ) ;
		u_char red_char, blue_char, green_char ; 
		red_char = ImageGetPixel( image, x, y, RED_CHANNEL ) ;
		green_char = ImageGetPixel( image, x, y, GREEN_CHANNEL ) ;
		blue_char = ImageGetPixel( image, x, y, BLUE_CHANNEL ) ;
		result.red = ( (double)red_char ) / 255 ;
		result.green = ( (double)green_char ) / 255 ;
		result.blue = ( (double)blue_char ) / 255 ;
		return result ;
}

// Maps parametric coordinates of an object onto texture property using the specified filter
Color Texture::map( double u, double v, int filter )
{
		if( !textureMap ) return Color( 0, 0, 0 ) ;
		switch( filter ) {

				case NO_FILTER:
						// Simply map (u,v) onto texel coords and return
						int xco, yco ;
						xco = (int)( ( textureMap -> width - 1 ) * u ) ; // Round on lower side generally
						yco = (int)( ( textureMap -> height - 1 ) * v ) ; // Round on lower side generally
						return getColorAtPixel( textureMap, xco, yco ) ;

				case BILINEAR_FILTER :
						Color result( 0, 0, 0 ) ;		
						// Find colors of surrounding texels and weight them according to frac. part of x,y
						double x, y ; int xtex, ytex ; 
						double h_wt, v_wt ;
						x = ( textureMap -> width - 1 ) * u ; 
						y = ( textureMap -> height - 1 ) * v ;
						xtex = (int)floor( x ) ; ytex = (int)floor( y ) ;
						h_wt = x - xtex ; v_wt = y - ytex ;
						double wt1, wt2, wt3, wt4 ;
						Color bottom_left, bottom_right, top_left, top_right ;

						bottom_left = getColorAtPixel( textureMap, xtex, ytex ) ;
						bottom_right = getColorAtPixel( textureMap, xtex + 1, ytex ) ;
						top_left = getColorAtPixel( textureMap, xtex, ytex + 1 ) ;
						top_right = getColorAtPixel( textureMap, xtex + 1, ytex + 1 ) ;
						wt1 = ( 1 - h_wt ) * ( 1 - v_wt ) ;
						wt2 = h_wt * ( 1 - v_wt ) ;
						wt3 = ( 1 - h_wt ) * v_wt ;
						wt4 = h_wt * v_wt ;

						// Compute result color using bilinear averaging
						result.red = wt1 * bottom_left.red + wt2 * bottom_right.red + wt3 * top_left.red + wt4 * top_right.red ;
						result.green = wt1 * bottom_left.green + wt2 * bottom_right.green + wt3 * top_left.green + wt4 * top_right.green ;
						result.blue = wt1 * bottom_left.blue + wt2 * bottom_right.blue + wt3 * top_left.blue + wt4 * top_right.blue ;
						return result ;
		}				// End of switch()
}	// End of map()
