// Defines the triangle - a derived shape from class object

#ifndef __TRIANGLE_H
#define __TRIANGLE_H

#include "object.h"
#include "primitive.h"
#include "surface.h"
#include "ray.h"
#include "csg.h"

// Supported triangle shading models
#define SHADE_FLAT 	0		// Flat shading
#define SHADE_PHONG 	1		// Phong shading (more computationally expensive)

class Triangle : public Primitive
{
	public:

	Vector3D vertex[3] ;
	Vector3D n[3] ; // In case of Phong shading

	Vector3D edge1, edge2 ; // Precompute these too to save time
	Vector3D normal ; // Precompute this to save some time
	
	int shading_model ; // The shading model to be used

	Triangle( Surface* s, const Vector3D& v, const Vector3D& v2, const Vector3D& v3 ) ;
	Triangle( Surface* s, const Vector3D& v1, const Vector3D& v2, const Vector3D& v3, const Vector3D& n1, const Vector3D& n2, const Vector3D& n3 ) ;
	Triangle( const Triangle& ) ;
	virtual bool intersect( Ray& ) ;
	virtual bool shadow_intersect( Ray& ) ;
	virtual void computeBounds( bVolume& b ) ; // Compute bounding volume for this triangle
	virtual Color shade( const Ray&, int depth ) ;
	virtual ~Triangle() {}

	// Compute interval list obtained by intersecting ray with triangle - not very useful in truth
	virtual vector <CSG_Interval*> compute_intervals( const Ray& ) ;
};

#endif
