// Class definition for 3D vectors - vector3D.cpp

#include "vector3D.h"
#include "wrappers.h"

#include <math.h>

Vector3D::Vector3D( double X, double Y, double Z )
{
	x = X ;
	y = Y ;
	z = Z ;
}

Vector3D::Vector3D( const Vector3D &v )
{
	
	x = v.x ;
	y = v.y ;
	z = v.z ;
}

/* Constructor from wrapper C struct */
Vector3D::Vector3D( const __wrap_Vector3D &v )
{
	x = v.x ;
	y = v.y ;
	z = v.z ;
}

// dot-product methods
double Vector3D::dot( const Vector3D &B ) const
{
	return ( x*B.x + y*B.y + z*B.z ) ;
}


double Vector3D::dot( double Bx, double By, double Bz ) const
{
	return ( x*Bx + y*By + z*Bz ) ;
}


// cross-product methods //FIXME copy constructor???
Vector3D Vector3D::cross( const Vector3D &B ) const
{
	Vector3D result ;
	result.x = y*B.z - z*B.y ;
	result.y = z*B.x - x*B.z ;
	result.z = x*B.y - y*B.x ;
	return result ;
}


Vector3D Vector3D::cross( double Bx, double By, double Bz ) const
{
	Vector3D result ;
	result.x = y*Bz - z*By ;
	result.y = z*Bx - x*Bz ;
	result.z = x*By - y*Bx ;
	return result ;
}


double Vector3D::length( ) const
{
	return ( (double)sqrt( x*x + y*y + z*z ) ) ;
}


double Vector3D::length( const Vector3D &A )
{
	return ( (double)sqrt( A.x*A.x + A.y*A.y + A.z*A.z ) ) ;
}


void Vector3D::normalize( )
{
	double length = Vector3D::length() ;
	x /= length ;
	y /= length ;
	z /= length ;
}


Vector3D Vector3D::normalize( const Vector3D &A )
{
	Vector3D result ;
	double length = A.length( ) ;
	result.x = A.x / length ;
	result.y = A.y / length ;
	result.z = A.z / length ;
	return result ;
}
